require 'securerandom'
require 'openssl'
require 'digest'
require 'base64'

#
# see: https://docs.github.com/en/developers/webhooks-and-events/securing-your-webhooks
#      https://github.com/square/connect-api-examples/blob/master/connect-examples/v1/ruby/webhooks.rb
#

module ExoBasic
  class HMACKeys
    SIZE = 20

    def self.gen_key(size=nil)
      s = size.nil? ? HMACKeys::SIZE : s

      SecureRandom.hex(s)
    end

    def self.sign_message(key, data)
      signature        = OpenSSL::HMAC.hexdigest('SHA256', key, data)
      signature_base64 = Base64.encode64(signature).gsub("\n", '')

      signature_base64
    end

    def self.verify_message(key, signature_base64, data)
      # Hash the signatures a second time (to protect against timing attacks) and compare them
      Digest::SHA256.base64digest(HMACKeys.sign_message(key, data)) ==
      Digest::SHA256.base64digest(signature_base64)
    end

  end
end
