module ExoBasic
  class LikertValue
    attr_reader :v, :maximum

    UNKNOWN = 0
    MINIMUM = 1

    def initialize(v, maximum)
      @v       = [LikertValue::UNKNOWN, [v, maximum].min].max
      @maximum = maximum
    end

    def get_range
      LikertValue::MINIMUM..@maximum
    end

    def get_extended_range
      LikertValue::UNKNOWN..@maximum
    end

    def normalize
      if @v != LikertValue::UNKNOWN
        [LikertValue::MINIMUM, [@v, @maximum].min].max
      else
        LikertValue::UNKNOWN
      end
    end

    def normalize_known
      [LikertValue::MINIMUM, [@v, @maximum].min].max
    end

    def self.real_to_likert(v, maximum)
      LikertValue.new(v.round(), maximum).normalize_known
    end

    def self.likert_avg(total_likert, n, maximum)
      LikertValue.real_to_likert(1.0 * total_likert / (1.0 * n), maximum)
    end

  end
end
