require 'logger'
require 'json'

module ExoBasic
  class Settings
    DEFAULT_FILENAME = './settings.json'

    @@app_name       = 'Untitled'
    @@load_observers = []

    def self.get_app_name
      @@app_name
    end
    def self.set_app_name(appname)
      @@app_name = appname
    end

    def self.add_on_load_observer(observer)
      @@load_observers.push(observer)
    end

    def self.post_load(js)
      merged = {}
      if js.key?('_include')
        to_include = []
        if js['_include'].is_a?(Array)
          to_include = js['_include']
        else
          to_include = [js['_include']]
        end

        to_include.each do |i|
          h = Settings.get_conf_with_rem(i)
          merged = merged.deep_merge(h)
        end
        merged = merged.deep_merge(js)
      else
        merged = js
      end

      merged
    end

    def self.get_conf_with_rem(filename=nil)
      f = filename
      if f.nil?
        f = Settings::DEFAULT_FILENAME
      end

      if File.exists?(f)
        js = JSON.load(
          File.readlines(f)
              .select { |line| !line.strip.start_with?('//') }
              .join())
        Settings.post_load(js)
      else
        {}
      end
    end

    def self.get_conf(filename=nil)
      f = filename
      if f.nil?
        f = Settings::DEFAULT_FILENAME
      end

      js = JSON.load(File.read(f))
      Settings.post_load(js)
    end

    def self.try_get_key(conf, default_value=false, k)
      conf.key?(k) ? conf[k] : default_value
    end

    def self.try_get_nested_key(conf, default_value=false, ks)
      if ks.empty?
        default_value
      else
        v = Settings.try_get_key(conf, default_value, ks[0])
        if v != default_value && ks.length > 1
          Settings.try_get_nested_key(v, default_value, ks[1..-1])
        else
          v
        end
      end
    end

    def self.load
      Settings.get_conf_with_rem
    end

    @@loaded = Settings.load

    def self.loaded
      @@loaded
    end

    def self.reload(filename=nil)
      @@loaded = Settings.get_conf_with_rem(filename)
      @@load_observers.each { |observer| observer.settings_reloaded }
    end

    def self.get_app_conf
      @@loaded[@@app_name]
    end

    def self.get_logger
      Logger.new(STDOUT)
    end

  end
end
