module ExoBasic
  class AvgMeta
    attr_reader :n,
                :change,
                :averageChange,
                :squareOfChange,
                :variance,
                :averageDirectional,
                :minPossibility,
                :maxPossibility

    def initialize
      @n                  = 0
      @change             = 0.0
      @averageChange      = 0.0
      @squareOfChange     = 0.0
      @variance           = 0.0
      @averageDirectional = 0.0
      @minPossibility     = Float::MAX
      @maxPossibility     = Float::MIN
    end

    def from_hash(h)
      @n                  = h[:n]
      @change             = h[:change]
      @averageChange      = h[:averageChange]
      @squareOfChange     = h[:squareOfChange]
      @variance           = h[:variance]
      @averageDirectional = h[:averageDirectional]
      @minPossibility     = h[:minPossibility]
      @maxPossibility     = h[:maxPossibility]
    end

    def self.mk(h)
      x = AvgMeta.new
      x.from_hash(h)

      x
    end

    def empty?
      @n == 0
    end

    def offer(x, prev_mean, d)
      delta                  = x - prev_mean
      d2                     = d <= 0 ? 1.0 : d.to_f
      change_prime           = delta.abs
      square_of_change_prime = delta * delta

      AvgMeta.mk({
        :n => @n + 1,
        :change => change_prime,
        :averageChange => change_prime / d2 + (d2 - 1.0) * @averageChange / d2,
        :squareOfChange => square_of_change_prime,
        :variance => square_of_change_prime / d2 + (d2 - 1.0) * @variance / d2,
        :averageDirectional => delta,
        :minPossibility => [x, @minPossibility].min,
        :maxPossibility => [x, @maxPossibility].max
      })
    end

    # Copy
    def deep_copy
      Marshal.load(Marshal.dump(self))
    end

    # Equal
    def ==(other)
      StatsHelpers.double_equals(@change,             other.change) &&
      StatsHelpers.double_equals(@variance,           other.variance) &&
      StatsHelpers.double_equals(@averageDirectional, other.averageDirectional) &&
      StatsHelpers.double_equals(@minPossibility,     other.minPossibility) &&
      StatsHelpers.double_equals(@maxPossibility,     other.maxPossibility)
    end

    # Monoid
    def self.zero
      AvgMeta.new
    end

    def self.append_helper(n1, d1, n2, d2, nAll)
      d1 * n1 / nAll + d2 * n2 / nAll
    end

    def +(other)
      n_prime = @n + other.n

      nd = n_prime.to_f
      n1 = @n.to_f
      n2 = other.n.to_f

      x = {
        :n => n_prime
      }
      if n1 <= 0
        x[:change]             = other.change
        x[:averageChange]      = other.averageChange
        x[:squareOfChange]     = other.squareOfChange
        x[:variance]           = other.variance
        x[:averageDirectional] = other.averageDirectional
      elsif n2 <= 0
        x[:change]             = @change
        x[:averageChange]      = @averageChange
        x[:squareOfChange]     = @squareOfChange
        x[:variance]           = @variance
        x[:averageDirectional] = @averageDirectional
      else
        x[:change]             = AvgMeta.append_helper(n1, @change,
                                                       n2, other.change,
                                                       nd),
        x[:averageChange]      = AvgMeta.append_helper(n1, @averageChange,
                                                       n2, other.averageChange,
                                                       nd),
        x[:squareOfChange]     = AvgMeta.append_helper(n1, @squareOfChange,
                                                       n2, other.squareOfChange,
                                                       nd),
        x[:variance]           = AvgMeta.append_helper(n1, @variance,
                                                       n2, other.variance,
                                                       nd),
        x[:averageDirectional] = AvgMeta.append_helper(n1, @averageDirectional,
                                                       n2, other.averageDirectional,
                                                       nd)
      end

      x[:minPossibility] = [@minPossibility, other.minPossibility].min
      x[:maxPossibility] = [@maxPossibility, other.maxPossibility].max

      AvgMeta.mk(x)
    end

    def standardDeviation
      Math.sqrt(@variance)
    end

    def possibilityRange
      [@minPossibility, @maxPossibility]
    end

  end
end
